#ifndef __FDLIST_H__
#define __FDLIST_H__

#include "ircd_config.h"

#ifdef HAVE_POLL

#include <string.h>
#include <poll.h>

typedef struct pollfd		FDLIST_POLLFD;

#else

#include <sys/types.h>
#include <sys/select.h>

#endif

#define FDLIST_FLAGS_READ	0x001
#define FDLIST_FLAGS_WRITE	0x002

typedef struct _fdlist		FDLIST;
typedef struct _fdlistinfo	FDLISTINFO;

struct _fdlist
{
#ifdef HAVE_POLL
	FDLIST_POLLFD	*pfd_array;
	int				pfd_size;
#else
	fd_set			write_fds;
	fd_set			read_fds;
	int				lowest_fd;
#endif
	int				pfd_num;
};

struct _fdlistinfo
{
    int             fd;
    FDLIST          *fdlist;
    int             fdlist_pos;
};

#ifdef HAVE_POLL

#define FDLIST_FD(__fdlist, __pos)	(__fdlist)->pfd_array[(__pos)].fd

#endif

#define SET_FDS(__fd, __ptr)	Fds[(__fd)] = (__ptr)
#define GET_FDS(__fd)			Fds[(__fd)]

#ifdef HAVE_POLL

#define fdlist_events(__fdlist, __pos)	(__fdlist)->pfd_array[(__pos)].events
#define fdlist_init(__fdlist)	memset((__fdlist), 0, sizeof(FDLIST))

#else

#define fdlist_init(__fdlist)	memset((__fdlist), 0, sizeof(FDLIST)),(__fdlist)->lowest_fd = (MAXCONNECTIONS+1)

#endif


int fdlist_add_fd(FDLIST *fdlist, int fd, int flags);
int fdlist_del_fd(FDLIST *fdlist, int pos, int *mfd);
int fdlist_add_event(FDLIST *fdlist, int pos, int flags);
int fdlist_del_event(FDLIST *fdlist, int pos, int flags);
int fdlist_set_events(FDLIST *fdlist, int pos, int flags);
int fdlist_get_events(FDLIST *fdlist, int pos);

#endif
